# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from ..services import blockchain_info, blockcypher_com, chain_com




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import json, requests, traceback

CHAIN_API_BASE_URL = 'https://api.chain.com/v2'

from .blockchain_client import BlockchainClient

class ChainComClient(BlockchainClient):
    def __init__(self, api_key_id=None, api_key_secret=None):
        self.type = 'chain.com'
        if api_key_id and api_key_secret:
            self.auth = (api_key_id, api_key_secret)
        else:
            self.auth = None

def format_unspents(unspents):
    return [{
        "transaction_hash": s["transaction_hash"],
        "output_index": s["output_index"],
        "value": s["value"],
        "script_opcodes": s["script"],
        "script_hex": s["script_hex"],
        "script_type": s["script_type"],
        "confirmations": s["confirmations"]
        }
        for s in unspents
    ]

def get_unspents(address, blockchain_client=ChainComClient()):
    """ Get the spendable transaction outputs, also known as UTXOs or
        unspent transaction outputs.
    """
    if not isinstance(blockchain_client, ChainComClient):
        raise Exception('A ChainComClient object is required')

    url = CHAIN_API_BASE_URL + '/bitcoin/addresses/' + address + '/unspents'

    auth = blockchain_client.auth
    if auth:
        r = requests.get(url, auth=auth)
    else:
        r = requests.get(url + '?api-key-id=DEMO-4a5e1e4')

    try:
        unspents = r.json()
    except ValueError, e:
        raise Exception('Received non-JSON response from chain.com.')
    
    return format_unspents(unspents)

def broadcast_transaction(hex_tx, blockchain_client):
    """ Dispatch a raw hex transaction to the network.
    """
    if not isinstance(blockchain_client, ChainComClient):
        raise Exception('A ChainComClient object is required')

    auth = blockchain_client.auth
    if not auth or len(auth) != 2:
        raise Exception('ChainComClient object must have auth credentials.')

    url = CHAIN_API_BASE_URL + '/bitcoin/transactions'
    payload = json.dumps({ 'hex': hex_tx })
    r = requests.put(url, data=payload, auth=auth)
    
    try:
        data = r.json()
    except ValueError, e:
        raise Exception('Received non-JSON from chain.com.')

    if 'transaction_hash' in data:
        data['success'] = True
        return data
    else:
        raise Exception('Tx hash missing from chain.com response: ' + str(data))




from .hash import bin_double_sha256, bin_to_hex_reversed, hex_to_bin_reversed

def hex_to_bin_reversed_hashes(hex_hashes):
    return [hex_to_bin_reversed(h) for h in hex_hashes]

def calculate_merkle_pairs(bin_hashes, hash_function=bin_double_sha256):
    """ takes in a list of binary hashes, returns a binary hash
    """
    hashes = list(bin_hashes)
    # if there are an odd number of hashes, double up the last one
    if len(hashes) % 2 == 1:
        hashes.append(hashes[-1])
    # build the new list of hashes
    new_hashes = []
    for i in range(0, len(hashes), 2):
        new_hashes.append(hash_function(hashes[i] + hashes[i+1]))
    # return the new list of hashes
    return new_hashes

def calculate_merkle_root(hashes, hash_function=bin_double_sha256, hex_format=True):
    """ takes in a list of binary hashes, returns a binary hash
    """
    if hex_format:
        hashes = hex_to_bin_reversed_hashes(hashes)
    # keep moving up the merkle tree, constructing one row at a time
    while len(hashes) > 1:
        hashes = calculate_merkle_pairs(hashes, hash_function)
    # get the merkle root
    merkle_root = hashes[0]
    # if the user wants the merkle root in hex format, convert it
    if hex_format:
        return bin_to_hex_reversed(merkle_root)
    # return the binary merkle root
    return merkle_root

class MerkleTree():
    def __init__(self, hashes, hex_format=True, hash_function=bin_double_sha256):
        if not len(hashes) > 0:
            raise ValueError("At least one hash is required.")

        self.rows = []

        # if the hashes are in hex format, first convert them to binary
        if hex_format:
            hashes = hex_to_bin_reversed_hashes(hashes)

        # build the rows of the merkle tree
        self.rows.append(hashes)
        while len(hashes) > 1:
            hashes = calculate_merkle_pairs(hashes, hash_function)
            self.rows.append(hashes)

    def get(self, row_index, column_index):
        # check to make sure there are enough rows
        if row_index + 1 > len(self.rows):
            raise ValueError("There aren't that many rows.")
        row = self.rows(row_index)
        # check to make sure there are enough items in the row
        if column_index + 1 > len(row):
            raise ValueError("There aren't that many items in that row.")
        # return the requested item
        return row[column_index]

    def root(self, hex_format=True):
        # return the merkle root
        bin_merkle_root = self.rows[-1][0]
        if hex_format:
            return bin_to_hex_reversed(bin_merkle_root)



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import json, requests, traceback
from ..hash import reverse_hash

BLOCKCHAIN_API_BASE_URL = "https://blockchain.info"

from .blockchain_client import BlockchainClient

class BlockchainInfoClient(BlockchainClient):
    def __init__(self, api_key=None):
        self.type = 'blockchain.info'
        if api_key:
            self.auth = (api_key, '')
        else:
            self.auth = None

def format_unspents(unspents):
    return [{
        "transaction_hash": reverse_hash(s["tx_hash"]),
        "output_index": s["tx_output_n"],
        "value": s["value"],
        "script_hex": s["script"],
        "confirmations": s["confirmations"]
        }
        for s in unspents
    ]

def get_unspents(address, blockchain_client=BlockchainInfoClient()):
    """ Get the spendable transaction outputs, also known as UTXOs or
        unspent transaction outputs.
    """
    if not isinstance(blockchain_client, BlockchainInfoClient):
        raise Exception('A BlockchainInfoClient object is required')

    url = BLOCKCHAIN_API_BASE_URL + "/unspent?format=json&active=" + address

    auth = blockchain_client.auth
    if auth and len(auth) == 2 and isinstance(auth[0], str):
        url = url + "&api_code=" + auth[0]

    r = requests.get(url, auth=auth)
    try:
        unspents = r.json()["unspent_outputs"]
    except ValueError, e:
        raise Exception('Invalid response from blockchain.info.')
    
    return format_unspents(unspents)

def broadcast_transaction(hex_tx, blockchain_client=BlockchainInfoClient()):
    """ Dispatch a raw transaction to the network.
    """
    url = BLOCKCHAIN_API_BASE_URL + '/pushtx'
    payload = {'tx': hex_tx}
    r = requests.post(url, data=payload, auth=blockchain_client.auth)
    
    if 'submitted' in r.text.lower():
        return {'success': True}
    else:
        raise Exception('Invalid response from blockchain.info.')





# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import re
from binascii import hexlify, unhexlify
from hashlib import sha256
from utilitybelt import change_charset
from .hash import bin_checksum

HEX_KEYSPACE = "0123456789abcdef"
B58_KEYSPACE = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"

def b58check_encode(bin_s, version_byte=0):
    """ Takes in a binary string and converts it to a base 58 check string. """
    # append the version byte to the beginning
    bin_s = chr(int(version_byte)) + bin_s
    # calculate the number of leading zeros
    num_leading_zeros = len(re.match(r'^\x00*', bin_s).group(0))
    # add in the checksum add the end
    bin_s = bin_s + bin_checksum(bin_s)
    # convert from b2 to b16
    hex_s = hexlify(bin_s)
    # convert from b16 to b58
    b58_s = change_charset(hex_s, HEX_KEYSPACE, B58_KEYSPACE)

    return B58_KEYSPACE[0] * num_leading_zeros + b58_s

def b58check_unpack(b58_s):
    """ Takes in a base 58 check string and returns: the version byte, the
        original encoded binary string, and the checksum.
    """
    num_leading_zeros = len(re.match(r'^1*', b58_s).group(0))
    # convert from b58 to b16
    hex_s = change_charset(b58_s, B58_KEYSPACE, HEX_KEYSPACE)
    # if an odd number of hex characters are present, add a zero to the front
    if len(hex_s) % 2 == 1:
        hex_s = "0" + hex_s
    # convert from b16 to b2
    bin_s = unhexlify(hex_s)
    # add in the leading zeros
    bin_s = '\x00' * num_leading_zeros + bin_s
    # make sure the newly calculated checksum equals the embedded checksum
    newly_calculated_checksum = bin_checksum(bin_s[:-4])
    embedded_checksum = bin_s[-4:]
    if not (newly_calculated_checksum == embedded_checksum):
        raise ValueError('b58check value has an invalid checksum')
    # return values
    version_byte = bin_s[:1]
    encoded_value = bin_s[1:-4]
    checksum = bin_s[-4:]
    return version_byte, encoded_value, checksum

def b58check_decode(b58_s):
    """ Takes in a base 58 check string and returns the original encoded binary
        string.
    """
    version_byte, encoded_value, checksum = b58check_unpack(b58_s)
    return encoded_value

def b58check_version_byte(b58_s):
    """ Takes in a base 58 check string and returns the version byte as an
        integer. """
    version_byte, encoded_value, checksum = b58check_unpack(b58_s)
    return ord(version_byte)

def is_b58check(b58_s):
    version_byte, binary_s, checksum = b58check_unpack(b58_s)
    return (b58_s == b58check_encode(binary_s,
        version_byte=ord(version_byte)))



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~
    
    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from inspect import isclass

from .keypair import *
from .passphrases import create_passphrase

def is_cryptocurrency_keypair_class(cls):
    if not isclass(cls):
        return False
    if cls.__name__ == 'BitcoinKeypair':
        return True
    if len(cls.__bases__) > 0 and cls.__bases__[0].__name__ == 'BitcoinKeypair':
        return True

_messages = {
    "SHORT_PASSPHRASE": "Warning! Passphrase must be at least %s characters.",
    "INVALID_KEYPAIR_CLASS": "Class must be a valid currency keypair class.",
}

class SDWallet():
    """ A sequential deterministic wallet.
    """

    def __init__(self, passphrase=None):
        """ Create wallet from a passphrase input. """
        if not passphrase:
            passphrase = create_passphrase(bits_of_entropy=160)

        self._passphrase = passphrase
    
    def passphrase(self):
        return self._passphrase

    def keypair(self, i, keypair_class):
        """ Return the keypair that corresponds to the provided sequence number
            and keypair class (BitcoinKeypair, etc.).
        """

        # Make sure keypair_class is a valid cryptocurrency keypair
        if not is_cryptocurrency_keypair_class(keypair_class):
            raise Exception(_messages["INVALID_KEYPAIR_CLASS"])

        currency_name = keypair_class.__name__.lower().replace('keypair', '')

        k = keypair_class.from_passphrase(
            self._passphrase + " " + currency_name + str(i))

        return k

class HDWallet():
    """ A hierarchical deterministic wallet in accordance with BIP 32.
    """

    def __init__(self):
        raise NotImplementedError()





# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import random, math
from .english_words import english_words_bip39, english_words_wiktionary, \
    english_words_google

system_random = random.SystemRandom()

def get_wordlist(language, word_source):
    """ Takes in a language and a word source and returns a matching wordlist,
        if it exists.
        Valid languages: ['english']
        Valid word sources: ['bip39', 'wiktionary', 'google']
    """
    try:
        wordlist_string = eval(language + '_words_' + word_source)
    except NameError:
        raise Exception("No wordlist could be found for the word source and language provided.")
    wordlist = wordlist_string.split(',')
    return wordlist

def get_num_words_with_entropy(bits_of_entropy, wordlist):
    """ Gets the number of words randomly selected from a given wordlist that
        would result in the number of bits of entropy specified.
    """
    entropy_per_word = math.log(len(wordlist))/math.log(2)
    num_words = int(math.ceil(bits_of_entropy/entropy_per_word))
    return num_words

def pick_random_words_from_wordlist(wordlist, num_words_to_choose):
    """ Picks words randomly from a wordlist.
    """
    return [system_random.choice(wordlist) for i in range(num_words_to_choose)]

def create_passphrase(bits_of_entropy=None, num_words=None,
                      language='english', word_source='wiktionary'):
    """ Creates a passphrase that has a certain number of bits of entropy OR
        a certain number of words.
    """
    wordlist = get_wordlist(language, word_source)

    if not num_words:
        if not bits_of_entropy:
            bits_of_entropy = 80
        num_words = get_num_words_with_entropy(bits_of_entropy, wordlist)

    return ' '.join(pick_random_words_from_wordlist(wordlist, num_words))

"""
def create_bip39_passphrase(bits_of_entropy):
    return create_passphrase(bits_of_entropy=bits_of_entropy, word_source='bip39')

def create_wiktionary_passphrase(bits_of_entropy):
    return create_passphrase(bits_of_entropy=bits_of_entropy, word_source='wiktionary')

def create_passphrase_with_num_words(num_words, language='english',
                                  word_source='bip39'):
    wordlist = get_wordlist(language, word_source)
    return ' '.join(pick_random_words_from_wordlist(wordlist, num_words))

def create_simple_passphrase():
    return create_passphrase_with_entropy(num_words=40, word_source='bip39')

def create_128bit_passphrase():
    return create_passphrase_with_entropy(128, word_source='bip39')

def create_160bit_passphrase():
    return create_passphrase_with_entropy(160, word_source='wiktionary')

def create_256bit_passphrase():
    return create_passphrase_with_entropy(256, word_source='wiktionary')
"""



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~
    
    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import os, re, random, binascii
from utilitybelt import is_int, is_hex

from .b58check import is_b58check

def is_secret_exponent(val, curve_order):
    return (isinstance(val, (int, long)) and val >= 1 and val < curve_order)

def is_256bit_hex_string(val):
    return (isinstance(val, str) and len(val) == 64 and is_hex(val))

def is_wif_pk(val):
    return (len(val) >= 51 and len(val) <= 51 and is_b58check(val))

def is_b58check_address(val):
    return is_b58check(val)
    #return (len(val) >= 27 and len(val) <= 34 and is_b58check(val))

def is_hex_ecdsa_pubkey(val):
    return (is_hex(val) and len(val) == 128)

def is_binary_ecdsa_pubkey(val):
	return (isinstance(val, str) and len(val) == 64)



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

OP_0 = 0
OP_PUSHDATA1 = 76 # 0x4c
OP_PUSHDATA2 = 77 # 0x4d
OP_PUSHDATA4 = 78 # 0x4e
OP_1NEGATE = 79
OP_RESERVED = 80
OP_1 = 81
OP_2 = 82
OP_3 = 83
OP_4 = 84
OP_5 = 85
OP_6 = 86
OP_7 = 87
OP_8 = 88
OP_9 = 89
OP_10 = 90
OP_11 = 91
OP_12 = 92
OP_13 = 93
OP_14 = 94
OP_15 = 95
OP_16 = 96
OP_NOP = 97
OP_VER = 98
OP_IF = 99
OP_NOTIF = 100
OP_VERIF = 101
OP_VERNOTIF = 102
OP_ELSE = 103
OP_ENDIF = 104
OP_VERIFY = 105 # 0x69
OP_RETURN = 106 # 0x6a
OP_TOALTSTACK = 107
OP_FROMALTSTACK = 108
OP_2DROP = 109
OP_2DUP = 110
OP_3DUP = 111
OP_2OVER = 112
OP_2ROT = 113
OP_2SWAP = 114
OP_IFDUP = 115
OP_DEPTH = 116
OP_DROP = 117
OP_DUP = 118
OP_NIP = 119
OP_OVER = 120
OP_PICK = 121
OP_ROLL = 122
OP_ROT = 123
OP_SWAP = 124
OP_TUCK = 125
OP_CAT = 126
OP_SUBSTR = 127
OP_LEFT = 128
OP_RIGHT = 129
OP_SIZE = 130
OP_INVERT = 131
OP_AND = 132
OP_OR = 133
OP_XOR = 134
OP_EQUAL = 135 # 0x87
OP_EQUALVERIFY = 136 # 0x88
OP_RESERVED1 = 137
OP_RESERVED2 = 138
OP_1ADD = 139
OP_1SUB = 140
OP_2MUL = 141
OP_2DIV = 142
OP_NEGATE = 143
OP_ABS = 144
OP_NOT = 145
OP_0NOTEQUAL = 146
OP_ADD = 147
OP_SUB = 148
OP_MUL = 149
OP_DIV = 150
OP_MOD = 151
OP_LSHIFT = 152
OP_RSHIFT = 153
OP_BOOLAND = 154
OP_BOOLOR = 155
OP_NUMEQUAL = 156
OP_NUMEQUALVERIFY = 157
OP_NUMNOTEQUAL = 158
OP_LESSTHAN = 159
OP_GREATERTHAN = 160
OP_LESSTHANOREQUAL = 161
OP_GREATERTHANOREQUAL = 162
OP_MIN = 163
OP_MAX = 164
OP_WITHIN = 165
OP_RIPEMD160 = 166
OP_SHA1 = 167
OP_SHA256 = 168
OP_HASH160 = 169 # 0xa9
OP_HASH256 = 170
OP_CODESEPARATOR = 171
OP_CHECKSIG = 172 # 0xac
OP_CHECKSIGVERIFY = 173 # 0xad
OP_CHECKMULTISIG = 174 # 0xae
OP_CHECKMULTISIGVERIFY = 175 # 0xaf
OP_NOP1 = 176
OP_NOP2 = 177
OP_NOP3 = 178
OP_NOP4 = 179
OP_NOP5 = 180
OP_NOP6 = 181
OP_NOP7 = 182
OP_NOP8 = 183
OP_NOP9 = 184
OP_NOP10 = 185
OP_PUBKEYHASH = 253 # 0xfd
OP_PUBKEY = 254 # 0xfe
OP_INVALIDOPCODE = 255



# print send_from_address_to_address('1BuD5zX3hgW7p74eouLWFWTgsaNpEo9mTG', '1BuD5zX3hgW7p74eouLWFWTgsaNpEo9mTG')
# 7790fa0ed25378512caa31aa5aad60e603868c09


"""

def standard_tx_fee():
    return 1000

def create_unsigned_tx(inputs, outputs,
                       fee=standard_tx_fee(), lock_time=0, version=1):
    pass

def sign_tx(unsigned_tx, private_keys):
    pass

def create_signed_tx(inputs, outputs, private_keys=[],
                     fee=standard_tx_fee(), lock_time=0, version=1):
    pass

class Input(object):
    def __init__(self, transaction_hash, output_index, script):
        self.transaction_hash = transaction_hash
        self.output_index = output_index
        self.script = script
        self.sequence = UINT_MAX

    def __str__(self):
        return self.to_dict()

    def to_dict(self):
        return {
            "transaction_hash": self.transaction_hash,
            "output_index": self.output_index,
            "script": self.script,
            "sequence": self.sequence
        }

    def serialize(self):
        return serialize_input(self.to_dict())

class Output(object):
    def __init__(self, value, script):
        self.value = value
        self.script = script

    def to_dict(self):
        return {
            "value": self.value,
            "script": self.script
        }

    def serialize(self):
        return serialize_output(self.to_dict())

"""




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import json, requests, traceback
from ..hash import reverse_hash

from bitcoinrpc.authproxy import AuthServiceProxy

from ..constants import SATOSHIS_PER_COIN

from binascii import unhexlify
from ..b58check import b58check_encode
from ..address import script_hex_to_address

from .blockchain_client import BlockchainClient

def create_bitcoind_service_proxy(
    rpc_username, rpc_password, server='127.0.0.1', port=8332, use_https=False):
    """ create a bitcoind service proxy
    """
    protocol = 'https' if use_https else 'http'
    uri = '%s://%s:%s@%s:%s' % (protocol, rpc_username, rpc_password,
        server, port)
    return AuthServiceProxy(uri)

class BitcoindClient(BlockchainClient):
    def __init__(self, rpc_username, rpc_password, use_https=False,
                 server='127.0.0.1', port=8332, version_byte=0):
        self.type = 'bitcoind'
        self.auth = (rpc_username, rpc_password)
        self.bitcoind = create_bitcoind_service_proxy(rpc_username,
            rpc_password, use_https=use_https, server=server, port=port)
        self.version_byte = version_byte

def format_unspents(unspents):
    return [{
        "transaction_hash": s["txid"],
        "output_index": s["vout"],
        "value": int(round(s["amount"]*SATOSHIS_PER_COIN)),
        "script_hex": s["scriptPubKey"],
        "confirmations": s["confirmations"]
        }
        for s in unspents
    ]

def get_unspents(address, blockchain_client):
    """ Get the spendable transaction outputs, also known as UTXOs or
        unspent transaction outputs.

        NOTE: this will only return unspents if the address provided is present
        in the bitcoind server. Use the blockchain or chain API to grab the
        unspents for arbitrary addresses.
    """
    if isinstance(blockchain_client, BitcoindClient):
        bitcoind = blockchain_client.bitcoind
        version_byte = blockchain_client.version_byte
    elif isinstance(blockchain_client, AuthServiceProxy):
        bitcoind = blockchain_client
        version_byte = 0
    else:
        raise Exception('A BitcoindClient object is required')

    all_unspents = bitcoind.listunspent()

    unspents = []
    for u in all_unspents:
        if 'address' not in u:
            u['address'] = script_hex_to_address(u['scriptPubKey'],
                                                 version_byte=version_byte)
        if 'spendable' in u and u['spendable'] is False:
            continue
        if u['address'] == address:
            unspents.append(u)

    return format_unspents(unspents)

def broadcast_transaction(hex_tx, blockchain_client):
    """ Dispatch a raw transaction to the network.
    """
    if isinstance(blockchain_client, BitcoindClient):
        bitcoind = blockchain_client.bitcoind
    elif isinstance(blockchain_client, AuthServiceProxy):
        bitcoind = blockchain_client
    else:
        raise Exception('A BitcoindClient object is required')

    resp = bitcoind.sendrawtransaction(hex_tx)
    if len(resp) > 0:
        return {'transaction_hash': resp, 'success': True}
    else:
        raise Exception('Invalid response from bitcoind.')




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import binascii, hashlib
from .formatcheck import is_hex

from .b58check import b58check_encode
from .errors import _errors

def bin_hash160(s):
    return hashlib.new('ripemd160', hashlib.sha256(s).digest()).digest()

class Hash160():
    def __init__(self, public_key, version_byte=0):
        if is_hex(public_key):
            binary_public_key = binascii.unhexlify(public_key)
        else:
            binary_public_key = public_key
        self._binary_value = bin_hash160(binary_public_key)
        self._version_byte = version_byte
    
    def to_bin(self):
        return self._binary_value
    
    def to_hex(self):
        return binascii.hexlify(self.to_bin())
    
    def to_b58check(self):
        return b58check_encode(self.to_bin(), version_byte=self._version_byte)
    
    def address(self):
        return self.to_b58check()
    
    def __str__(self):
        return self.to_hex()

    def __repr__(self):
        return self.to_hex()

"""class PubkeyFormat():
    bin_ecdsa = 1
    hex_ecdsa = 2
    bin_uncompressed = 3
    hex_uncompressed = 4
    bin_compressed = 5
    hex_compressed = 6
"""

""" The address is the hash160 in b58check format. """

"""if hasattr(self, '_compressed_bin_public_key'):
    pubkey = self._compressed_bin_public_key
else:
    pubkey = self.to_bin()
"""
#return pubkey_to_address(pubkey, magicbyte=self._version_byte)

"""
# if the public key is compressed, decompress it
if is_hex(public_key) and len(public_key) == 66:
    public_key = decompress(public_key)
    self._compressed = True
elif len(public_key) == 33:
    public_key = decompress(public_key)
    self._compressed = True

# strip away the \x04 byte if it is present
if is_hex(public_key) and len(public_key) == 130 and public_key[0:2] == '04':
    public_key = public_key[2:]
elif len(public_key) == 65 and public_key[0] == '\x04':
    public_key = public_key[1:]

# get the binary version of the public key
if is_hex_ecdsa_pubkey(public_key):
    bin_public_key = unhexlify(public_key)
elif is_binary_ecdsa_pubkey(public_key):
    bin_public_key = public_key
else:
    raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])
"""



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~
    
    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import os, json, hashlib, ecdsa
from binascii import hexlify, unhexlify
from ecdsa.keys import SigningKey
from utilitybelt import is_int, dev_random_entropy, dev_urandom_entropy
from pybitcointools import compress

from .errors import _errors
from .formatcheck import *
from .b58check import b58check_encode, b58check_decode
from .publickey import BitcoinPublicKey, PUBKEY_MAGIC_BYTE
from .passphrases import create_passphrase

def random_secret_exponent(curve_order):
    """ Generates a random secret exponent. """
    # run a rejection sampling algorithm to ensure the random int is less
    # than the curve order
    while True:
        # generate a random 256 bit hex string
        random_hex = hexlify(dev_random_entropy(32))
        random_int = int(random_hex, 16)
        if random_int >= 1 and random_int < curve_order:
            break
    return random_int

class BitcoinPrivateKey():
    _curve = ecdsa.curves.SECP256k1
    _hash_function = hashlib.sha256
    _pubkeyhash_version_byte = 0

    @classmethod
    def wif_version_byte(cls):
        if hasattr(cls, '_wif_version_byte'):
            return cls._wif_version_byte
        return (cls._pubkeyhash_version_byte + 128) % 256

    def __init__(self, private_key=None):
        """ Takes in a private key/secret exponent.
        """
        if not private_key:
            secret_exponent = random_secret_exponent(self._curve.order)
        else:
            private_key = str(private_key)

            if is_int(private_key):
                secret_exponent = private_key
            elif is_256bit_hex_string(private_key):
                secret_exponent = int(private_key, 16)
            elif is_wif_pk(private_key):
                secret_exponent = int(hexlify(b58check_decode(private_key)), 16)

        # make sure that: 1 <= secret_exponent < curve_order
        if not is_secret_exponent(secret_exponent, self._curve.order):
            raise IndexError(_errors["EXPONENT_OUTSIDE_CURVE_ORDER"])
        
        self._ecdsa_private_key = ecdsa.keys.SigningKey.from_secret_exponent(
            secret_exponent, self._curve, self._hash_function
        )

    @classmethod
    def from_passphrase(cls, passphrase=None):
        """ Create keypair from a passphrase input (a brain wallet keypair)."""
        if not passphrase:
            # run a rejection sampling algorithm to ensure the private key is
            # less than the curve order
            while True:
                passphrase = create_passphrase(bits_of_entropy=160)
                hex_private_key = hashlib.sha256(passphrase).hexdigest()
                if int(hex_private_key, 16) < cls._curve.order:
                    break
        else:
            hex_private_key = hashlib.sha256(passphrase).hexdigest()
            if not (int(hex_private_key, 16) < cls._curve.order):
                raise ValueError(_errors["CURVE_ORDER_EXCEEDED"])

        keypair = cls(hex_private_key)
        keypair._passphrase = passphrase
        return keypair

    def to_bin(self):
        return self._ecdsa_private_key.to_string()

    def to_hex(self):
        return hexlify(self.to_bin())

    def to_wif(self):
        return b58check_encode(self.to_bin(),
            version_byte=self.wif_version_byte())

    def public_key(self, compressed=False):
        # lazily calculate and set the public key
        if not hasattr(self, '_public_key'):            
            ecdsa_public_key = self._ecdsa_private_key.get_verifying_key()

            # compress the public key string if a compressed public key is desired
            bin_public_key_string = PUBKEY_MAGIC_BYTE + ecdsa_public_key.to_string()
            if compressed:
                bin_public_key_string = compress(bin_public_key_string)
            # create the public key object from the public key string
            self._public_key = BitcoinPublicKey(bin_public_key_string,
                version_byte=self._pubkeyhash_version_byte)
        
        # return the public key object
        return self._public_key

    def passphrase(self):
        if hasattr(self, '_passphrase'):
            return self._passphrase
        else:
            raise Exception(_errors["NOT_A_BRAIN_WALLET"])

class LitecoinPrivateKey(BitcoinPrivateKey):
    _pubkeyhash_version_byte = 48

class NamecoinPrivateKey(BitcoinPrivateKey):
    _pubkeyhash_version_byte = 52




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import struct
from binascii import hexlify, unhexlify
from utilitybelt import is_hex

def count_bytes(hex_s):
    """ Calculate the number of bytes of a given hex string.
    """
    assert(is_hex(hex_s))
    return len(hex_s)/2

def flip_endian(s):
    if is_hex:
        return hexlify(unhexlify(s)[::-1])
    return s[::-1]

def variable_length_int(i):
    """ Encodes integers into variable length integers, which are used in
        Bitcoin in order to save space.
    """
    if not isinstance(i, (int,long)):
        raise Exception('i must be an integer')

    if i < (2**8-3):
        return chr(i) # pack the integer into one byte
    elif i < (2**16):
        return chr(253) + struct.pack('<H', i) # pack into 2 bytes
    elif i < (2**32):
        return chr(254) + struct.pack('<I', i) # pack into 4 bytes
    elif i < (2**64):
        return chr(255) + struct.pack('<Q', i) # pack into 8 bites
    else:
        raise Exception('Integer cannot exceed 8 bytes in length.')


# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from binascii import hexlify, unhexlify
from pybitcointools import sign as sign_transaction

from ..services import blockchain_info, chain_com, bitcoind
from ..privatekey import BitcoinPrivateKey
from .serialize import serialize_transaction
from .outputs import make_pay_to_address_outputs, make_op_return_outputs
from ..constants import STANDARD_FEE, OP_RETURN_FEE

""" Note: for functions that take in an auth object, here are some examples
    for the various APIs available:
    
    blockchain.info: auth=(api_key, None)
    chain.com: auth=(api_key_id, api_key_secret)
"""

from ..services import BlockchainInfoClient, BitcoindClient, ChainComClient, \
    BlockchainClient
from bitcoinrpc.authproxy import AuthServiceProxy

def get_unspents(address, blockchain_client=BlockchainInfoClient()):
    """ Gets the unspent outputs for a given address.
    """
    if isinstance(blockchain_client, BlockchainInfoClient):
        return blockchain_info.get_unspents(address, blockchain_client)
    elif isinstance(blockchain_client, ChainComClient):
        return chain_com.get_unspents(address, blockchain_client)
    elif isinstance(blockchain_client, (BitcoindClient, AuthServiceProxy)):
        return bitcoind.get_unspents(address, blockchain_client)
    elif isinstance(blockchain_client, BlockchainClient):
        raise Exception('That blockchain interface is not supported.')
    else:
        raise Exception('A BlockchainClient object is required')

def broadcast_transaction(hex_tx, blockchain_client):
    """ Dispatches a raw hex transaction to the network.
    """
    if isinstance(blockchain_client, BlockchainInfoClient):
        return blockchain_info.broadcast_transaction(hex_tx, blockchain_client)
    elif isinstance(blockchain_client, ChainComClient):
        return chain_com.broadcast_transaction(hex_tx, blockchain_client)
    elif isinstance(blockchain_client, (BitcoindClient, AuthServiceProxy)):
        return bitcoind.broadcast_transaction(hex_tx, blockchain_client)
    elif isinstance(blockchain_client, BlockchainClient):
        raise Exception('That blockchain interface is not supported.')
    else:
        raise Exception('A BlockchainClient object is required')

def get_private_key_obj(private_key):
    if isinstance(private_key, BitcoinPrivateKey):
        return private_key
    else:
        return BitcoinPrivateKey(private_key)

def analyze_private_key(private_key, blockchain_client):
    private_key_obj = get_private_key_obj(private_key)
    # determine the address associated with the supplied private key
    from_address = private_key_obj.public_key().address()
    # get the unspent outputs corresponding to the given address
    inputs = get_unspents(from_address, blockchain_client)
    # return the inputs
    return private_key_obj, from_address, inputs

def make_send_to_address_tx(recipient_address, amount, private_key,
        blockchain_client=BlockchainInfoClient(), fee=STANDARD_FEE,
        change_address=None):
    """ Builds and signs a "send to address" transaction.
    """
    # get out the private key object, sending address, and inputs
    private_key_obj, from_address, inputs = analyze_private_key(private_key,
        blockchain_client)
    # get the change address
    if not change_address:
        change_address = from_address
    # create the outputs
    outputs = make_pay_to_address_outputs(recipient_address, amount, inputs,
                                          change_address, fee=fee)
    # serialize the transaction
    unsigned_tx = serialize_transaction(inputs, outputs)
    # sign the unsigned transaction with the private key
    signed_tx = sign_transaction(unsigned_tx, 0, private_key_obj.to_hex())
    # return the signed tx
    return signed_tx

def make_op_return_tx(data, private_key,
        blockchain_client=BlockchainInfoClient(), fee=OP_RETURN_FEE,
        change_address=None, format='bin'):
    """ Builds and signs an OP_RETURN transaction.
    """
    # get out the private key object, sending address, and inputs
    private_key_obj, from_address, inputs = analyze_private_key(private_key,
        blockchain_client)
    # get the change address
    if not change_address:
        change_address = from_address
    # create the outputs
    outputs = make_op_return_outputs(data, inputs, change_address,
        fee=fee, format=format)
    # serialize the transaction
    unsigned_tx = serialize_transaction(inputs, outputs)
    # sign the unsigned transaction with the private key
    signed_tx = sign_transaction(unsigned_tx, 0, private_key_obj.to_hex())
    # return the signed tx
    return signed_tx

def send_to_address(recipient_address, amount, private_key,
        blockchain_client=BlockchainInfoClient(), fee=STANDARD_FEE,
        change_address=None):
    """ Builds, signs, and dispatches a "send to address" transaction.
    """
    # build and sign the tx
    signed_tx = make_send_to_address_tx(recipient_address, amount,
        private_key, blockchain_client, fee=fee,
        change_address=change_address)
    # dispatch the signed transction to the network
    response = broadcast_transaction(signed_tx, blockchain_client)
    # return the response
    return response

def embed_data_in_blockchain(data, private_key,
        blockchain_client=BlockchainInfoClient(), fee=OP_RETURN_FEE,
        change_address=None, format='bin'):
    """ Builds, signs, and dispatches an OP_RETURN transaction.
    """
    # build and sign the tx
    signed_tx = make_op_return_tx(data, private_key, blockchain_client,
        fee=fee, change_address=change_address, format=format)
    # dispatch the signed transction to the network
    response = broadcast_transaction(signed_tx, blockchain_client)
    # return the response
    return response

def serialize_sign_and_broadcast(inputs, outputs, private_key,
                                 blockchain_client=BlockchainInfoClient()):
    # extract the private key object
    private_key_obj = get_private_key_obj(private_key)
    # serialize the transaction
    unsigned_tx = serialize_transaction(inputs, outputs)
    # sign the unsigned transaction with the private key
    signed_tx = sign_transaction(unsigned_tx, 0, private_key_obj.to_hex())
    # dispatch the signed transction to the network
    response = broadcast_transaction(signed_tx, blockchain_client)
    # return the response
    return response



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import json, requests, traceback

BLOCKCYPHER_API_BASE_URL = "http://api.blockcypher.com/v1/btc/main"

def format_unspents(unspents):
    return [{
        "transaction_hash": s["tx_hash"],
        "output_index": s["tx_output_n"],
        "value": s["value"],
        "confirmations": s["confirmations"]
        }
        for s in unspents
    ]

def get_unspents(address, auth=None):
    """ Get the spendable transaction outputs, also known as UTXOs or
        unspent transaction outputs.
    """
    url = BLOCKCYPHER_API_BASE_URL + "/addrs/" + address + "?unspentOnly=true"

    r = requests.get(url)
    try:
        data = r.json()
    except ValueError, e:
        raise Exception('Invalid response from API.')

    unspents = []
    if "txrefs" in data:
        unspents.extend(data["txrefs"])
    if "unconfirmed_txrefs" in data:
        unspents.extend(data["unconfirmed_txrefs"])

    return format_unspents(unspents)

class BlockcypherClient():
    def __init__(self):
        pass

    def auth(self):
        return None

    def get_unspents(self, address):
        return get_unspents(address)

if __name__ == '__main__':
    pass




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~
    
    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import os, json, hashlib, ecdsa
from binascii import hexlify, unhexlify
from ecdsa.keys import VerifyingKey
from pybitcointools import decompress, compress, pubkey_to_address
from utilitybelt import is_hex

from .errors import _errors
from .hash import bin_hash160 as get_bin_hash160
from .formatcheck import is_hex_ecdsa_pubkey, is_binary_ecdsa_pubkey
from .b58check import b58check_encode
from .address import bin_hash160_to_address

PUBKEY_MAGIC_BYTE = '\x04'

class CharEncoding():
    hex = 16
    bin = 256

class PubkeyType():
    ecdsa = 1
    uncompressed = 2
    compressed = 3

def get_public_key_format(public_key_string):
    if not isinstance(public_key_string, str):
        raise ValueError('Public key must be a string.')

    if len(public_key_string) == 64:
        return CharEncoding.bin, PubkeyType.ecdsa
    if len(public_key_string) == 65 and public_key_string[0] == PUBKEY_MAGIC_BYTE:
        return CharEncoding.bin, PubkeyType.uncompressed
    if len(public_key_string) == 33:
        return CharEncoding.bin, PubkeyType.compressed
    
    if is_hex(public_key_string):
        if len(public_key_string) == 128:
            return CharEncoding.hex, PubkeyType.ecdsa
        if len(public_key_string) == 130 and public_key_string[0:2] == hexlify(PUBKEY_MAGIC_BYTE):
            return CharEncoding.hex, PubkeyType.uncompressed
        if len(public_key_string) == 66:
            return CharEncoding.hex, PubkeyType.compressed

    raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])

def extract_bin_ecdsa_pubkey(public_key):
    key_charencoding, key_type = get_public_key_format(public_key)

    if key_charencoding == CharEncoding.hex:
        bin_public_key = unhexlify(public_key)
    elif key_charencoding == CharEncoding.bin:
        bin_public_key = public_key
    else:
        raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])

    if key_type == PubkeyType.ecdsa:
        return bin_public_key
    elif key_type == PubkeyType.uncompressed:
        return bin_public_key[1:]
    elif key_type == PubkeyType.compressed:
        return decompress(bin_public_key)[1:]
    else:
        raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])

def extract_bin_bitcoin_pubkey(public_key):
    key_charencoding, key_type = get_public_key_format(public_key)

    if key_charencoding == CharEncoding.hex:
        bin_public_key = unhexlify(public_key)
    elif key_charencoding == CharEncoding.bin:
        bin_public_key = public_key
    else:
        raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])

    if key_type == PubkeyType.ecdsa:
        return PUBKEY_MAGIC_BYTE + bin_public_key
    elif key_type == PubkeyType.uncompressed:
        return bin_public_key
    elif key_type == PubkeyType.compressed:
        return bin_public_key
    else:
        raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])

class BitcoinPublicKey():
    _curve = ecdsa.curves.SECP256k1
    _version_byte = 0

    @classmethod
    def version_byte(cls):
        return cls._version_byte

    def __init__(self, public_key_string, version_byte=0, verify=True):
        """ Takes in a public key in hex format.
        """
        # set the version byte
        self._version_byte = version_byte
        self._charencoding, self._type = get_public_key_format(public_key_string)

        # extract the binary bitcoin key (compressed or uncompressed w/ magic byte)
        self._bin_public_key = extract_bin_bitcoin_pubkey(public_key_string)
        
        # extract the bin ecdsa public key (uncompressed, w/out a magic byte)
        bin_ecdsa_public_key = extract_bin_ecdsa_pubkey(public_key_string)
        if verify:
            try:
                # create the ecdsa key object
                self._ecdsa_public_key = VerifyingKey.from_string(
                    bin_ecdsa_public_key, self._curve)
            except AssertionError as e:
                raise ValueError(_errors['IMPROPER_PUBLIC_KEY_FORMAT'])

    def to_bin(self):
        return self._bin_public_key

    def to_hex(self):
        return hexlify(self.to_bin())

    def bin_hash160(self):
        if not hasattr(self, '_bin_hash160'):
            self._bin_hash160 = get_bin_hash160(self.to_bin())
        return self._bin_hash160

    def hash160(self):
        return hexlify(self.bin_hash160())

    def address(self):
        return bin_hash160_to_address(self.bin_hash160(),
                                      version_byte=self._version_byte)

class LitecoinPublicKey(BitcoinPublicKey):
    _version_byte = 48

class NamecoinPublicKey(BitcoinPublicKey):
    _version_byte = 52



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from .blockchain_client import BlockchainClient
from blockchain_info import BlockchainInfoClient
from chain_com import ChainComClient
from bitcoind import BitcoindClient, create_bitcoind_service_proxy

import blockchain_info
import chain_com
import bitcoind




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

_errors = {
	"EXPONENT_OUTSIDE_CURVE_ORDER": ("Secret exponent is outside of the"
		"valid range. Must be >= 1 and < the curve order."),
    "PHRASE_YIELDS_INVALID_EXPONENT": ("Invalid passphrase. The hash of this "
        "passphrase exceeds the curve order. Please try another passphrase."),
    "MUST_BE_VALID_PRIVKEY_FORMAT": ("Format must be bin, hex, wif, or "
        "b58check."),
    "MUST_BE_VALID_PUBKEY_FORMAT": ("Format must be bin or hex"),
    "MUST_BE_VALID_HASH160_FORMAT": ("format must be bin, hex or "
        "b58check."),
    "NOT_A_BRAIN_WALLET": ("No passphrase! This isn't a brain wallet address!"),
    "MUST_BE_A_HEX_STRING": ("Must be a hex string"),
    "IMPROPER_PUBLIC_KEY_FORMAT": ("Public key is not in proper format"),
}


# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

# data structure constants
UINT_MAX = 2**32-1

# protocol constants
SATOSHIS_PER_COIN = 10**8
MAX_BYTES_AFTER_OP_RETURN = 40

# fee defaults
STANDARD_FEE = 1000 # 1000 satoshis = 10 bits = .01 mbits = .00001 BTC
OP_RETURN_FEE = 10000 # 10k satoshis = .0001 BTC



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

class BlockchainClient(object):
    """ type parameter can be 'bitcoind', 'blockchain.info', 'chain.com', etc.
        auth object is a two item tuple
    """
    def __init__(self, type, auth=None):
        self.type = type
        if isinstance(auth, tuple) and len(auth) == 2:
            self.auth = auth
        else:
            raise Exception('auth must be a two-item tuple')



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from .scripts import make_pay_to_address_script, make_op_return_script
from ..constants import STANDARD_FEE, OP_RETURN_FEE

def calculate_change_amount(inputs, send_amount, fee):
    # calculate the total amount  coming into the transaction from the inputs
    total_amount_in = sum([input['value'] for input in inputs])
    # change = whatever is left over from the amount sent & the transaction fee
    change_amount = total_amount_in - send_amount - fee
    # check to ensure the change amount is a non-negative value and return it
    if change_amount < 0:
        raise Exception('Not enough inputs for transaction.')
    return change_amount

def make_pay_to_address_outputs(to_address, send_amount, inputs, change_address,
                                fee=STANDARD_FEE):
    """ Builds the outputs for a "pay to address" transaction.
    """
    return [
        # main output
        { "script_hex": make_pay_to_address_script(to_address), "value": send_amount },
        # change output
        { "script_hex": make_pay_to_address_script(change_address),
          "value": calculate_change_amount(inputs, send_amount, fee)
        }
    ]

def make_op_return_outputs(data, inputs, change_address, fee=OP_RETURN_FEE,
                           send_amount=0, format='bin'):
    """ Builds the outputs for an OP_RETURN transaction.
    """
    return [
        # main output
        { "script_hex": make_op_return_script(data, format=format), "value": send_amount },
        # change output
        { "script_hex": make_pay_to_address_script(change_address),
          "value": calculate_change_amount(inputs, send_amount, fee)
        }
    ]



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import opcodes

from .network import broadcast_transaction, send_to_address, get_unspents, \
    embed_data_in_blockchain, make_send_to_address_tx, make_op_return_tx, \
    analyze_private_key, serialize_sign_and_broadcast
from .scripts import make_pay_to_address_script, make_op_return_script, \
    script_to_hex
from .serialize import serialize_input, serialize_output, serialize_transaction
from .outputs import make_op_return_outputs, make_pay_to_address_outputs
from .utils import flip_endian, variable_length_int



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from binascii import hexlify, unhexlify
import struct
from .utils import flip_endian, variable_length_int
from ..constants import UINT_MAX

def serialize_input(input, signature_script_hex=''):
    """ Serializes a transaction input.
    """
    if not (isinstance(input, dict) and 'transaction_hash' in input \
            and 'output_index' in input):
        raise Exception('Required parameters: transaction_hash, output_index')

    if not 'sequence' in input:
        input['sequence'] = UINT_MAX

    return ''.join([
        flip_endian(input['transaction_hash']),
        hexlify(struct.pack('<I', input['output_index'])),
        hexlify(variable_length_int(len(signature_script_hex)/2)),
        signature_script_hex,
        hexlify(struct.pack('<I', input['sequence']))
    ])

def serialize_output(output):
    """ Serializes a transaction output.
    """
    if not ('value' in output and 'script_hex' in output):
        raise Exception('Invalid output')

    return ''.join([
        hexlify(struct.pack('<Q', output['value'])), # pack into 8 bites
        hexlify(variable_length_int(len(output['script_hex'])/2)),
        output['script_hex']
    ])

def serialize_transaction(inputs, outputs, lock_time=0, version=1):
    """ Serializes a transaction.
    """

    # add in the inputs
    serialized_inputs = ''.join([serialize_input(input) for input in inputs])

    # add in the outputs
    serialized_outputs = ''.join([serialize_output(output) for output in outputs])

    return ''.join([
        # add in the version number
        hexlify(struct.pack('<I', version)),
        # add in the number of inputs
        hexlify(variable_length_int(len(inputs))),
        # add in the inputs
        serialized_inputs,
        # add in the number of outputs
        hexlify(variable_length_int(len(outputs))),
        # add in the outputs
        serialized_outputs,
        # add in the lock time
        hexlify(struct.pack('<I', lock_time)),
    ])



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import os, binascii
from utilitybelt import dev_random_entropy, dev_urandom_entropy

def random_secret_exponent(curve_order):
    """ Generates a random secret exponent. """
    # run a rejection sampling algorithm to ensure the random int is less
    # than the curve order
    while True:
        # generate a random 256 bit hex string
        random_hex = binascii.hexlify(dev_random_entropy(32))
        random_int = int(random_hex, 16)
        if random_int >= 1 and random_int < curve_order:
            break
    return random_int



import traceback, json
from coinkit import *
from collections import defaultdict
from pprint import pprint
from decimal import *

class ExtendedJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, Decimal):
            return float(o)
        return super(ExtendedJSONEncoder, self).default(o)

class Blockchain(object):
    def __init__(self, bitcoind_client):
        self.txs = {}
        self.address_txs = defaultdict(dict)
        self.bitcoind = bitcoind_client.bitcoind
        self.block_count = self.bitcoind.getblockcount()
        self.nulldata_txs = defaultdict(list)

    def spend_input(self, tx_hash, input):
        if 'txid' in input and 'vout' in input and 'scriptSig' in input:
            input_tx_hash = str(input['txid'])
            input_index = str(input['vout'])
            
            script_sig_asm = str(input['scriptSig'].get('asm'))
            script_sig_hex = str(input['scriptSig'].get('hex'))
            script_sig_parts = script_sig_asm.split(' ')
            if len(script_sig_parts) > 1 and (len(script_sig_parts[-1]) == 130 
                or len(script_sig_parts[-1]) == 66):
                public_key_string = script_sig_parts[-1]
                try:
                    recipient_address = BitcoinPublicKey(public_key_string, verify=False).address()
                except:
                    print "Error with transaction..."
                    print "Input tx hash: %s" % input_tx_hash
                    print "Current tx hash: %s" % tx_hash
                    print input['scriptSig']
                    traceback.print_exc()
                    return
            elif len(script_sig_parts) == 1:
                print "coinbase transaction input"
                return
            else:
                print "input with unknown transaction type"
                print "Input tx hash: %s" % input_tx_hash
                print "Current tx hash: %s" % tx_hash
                print input['scriptSig']
                return

            #print recipient_address
            input_id = '%s-%s' % (input_tx_hash, input_index)

            if input_id in self.address_txs[recipient_address]:
                self.address_txs[recipient_address][input_id] = False

    def log_output(self, tx_hash, output):
        if 'scriptPubKey' in output and 'n' in output:
            output_index = output['n']
            output_script = output['scriptPubKey']
            if 'addresses' in output_script:
                output_addresses = output_script['addresses']
                for address in output_addresses:
                    self.address_txs[address]['%s-%s' % (tx_hash, output_index)] = True

    def check_for_nulldata(self, tx, block_number):
        if 'vin' in tx and 'vout' in tx:
            inputs = tx['vin']
            outputs = tx['vout']
            for output in outputs:
                if 'scriptPubKey' in output:
                    script_asm = str(output['scriptPubKey'].get('asm'))
                    script_parts = script_asm.split(' ')
                    script_type = str(output['scriptPubKey'].get('type'))
                    if script_type == 'nulldata' and len(script_parts) == 2:
                        script_data = script_parts[1]
                        nulldata_tx = {'data': script_data, 'outputs': outputs }
                        self.nulldata_txs[block_number].append(nulldata_tx)
                        print "nulldata tx"

    def process_tx(self, tx_hash, block_number):
        print tx_hash
        # lookup the raw tx using the tx hash
        try:
            raw_tx = self.bitcoind.getrawtransaction(tx_hash)
        except:
            traceback.print_exc()
            return
        
        # extract the tx data from the raw tx
        tx = self.bitcoind.decoderawtransaction(raw_tx)
        # update tx index
        self.txs[tx_hash] = tx
        # log all the outputs in the transaction
        if 'vout' in tx:
            outputs = tx['vout']
            for output in outputs:
                self.log_output(tx_hash, output)
        # all the inputs were once unspent outputs, but now they are spent...
        # make sure they are marked as such
        if 'vin' in tx:
            inputs = tx['vin']
            for input in inputs:
                self.spend_input(tx_hash, input)

        self.check_for_nulldata(tx, block_number)

    def index(self, first_block=None, last_block=None):
        if not first_block and last_block:
            block_count = self.bitcoind.getblockcount()
            first_block, last_block = 0, block_count

        for block_number in range(first_block, last_block+1):
            print "\n%s%s%s" % ("="*10, str(block_number), "="*10)
            block_hash = self.bitcoind.getblockhash(block_number)
            block_data = self.bitcoind.getblock(block_hash)
            if 'tx' in block_data:
                tx_hashes = block_data['tx']
                for tx_hash in tx_hashes:
                    self.process_tx(tx_hash, block_number)

    def save_address_txs(self, filename):
        with open(filename, 'w') as f:
            f.write(json.dumps(self.address_txs, cls=ExtendedJSONEncoder))

    def save_nulldata_txs(self, filename):
        with open(filename, 'w') as f:
            f.write(json.dumps(self.nulldata_txs, cls=ExtendedJSONEncoder))

def get_bitcoind_client():
    try:
        with open('tests/secrets.json', 'r') as f:
            SECRETS = json.loads(f.read())
    except Exception as e:
        traceback.print_exc()
        return None

    bitcoind_client = BitcoindClient(SECRETS['rpc_username'], SECRETS['rpc_password'])
    return bitcoind_client

from datetime import datetime

def main():
    start = datetime.now()
    first_block, num_blocks = 331757, 2
    bitcoind_client = get_bitcoind_client()
    if bitcoind_client:
        block_count = bitcoind_client.bitcoind.getblockcount()
        blockchain = Blockchain(bitcoind_client)
        blockchain.index(first_block, first_block+num_blocks-1)
        blockchain.save_address_txs('ignore/address_txs.txt')
        blockchain.save_nulldata_txs('ignore/nulldata_txs.txt')
    end = datetime.now()
    delta = end - start
    print "%s seconds" % delta.seconds

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import hashlib
from hashlib import sha256
from binascii import hexlify, unhexlify
from utilitybelt import is_hex

def bin_sha256(bin_s):
    return sha256(bin_s).digest()

def bin_checksum(bin_s):
    """ Takes in a binary string and returns a checksum. """
    return bin_sha256(bin_sha256(bin_s))[:4]

def bin_double_sha256(bin_s):
    return bin_sha256(bin_sha256(bin_s))

def bin_hash160(s, hex_format=False):
    """ s is in hex or binary format
    """
    if hex_format and is_hex(s):
        s = unhexlify(s)
    return hashlib.new('ripemd160', bin_sha256(s)).digest()

def hex_hash160(s, hex_format=False):
    """ s is in hex or binary format
    """
    if hex_format and is_hex(s):
        s = unhexlify(s)
    return hexlify(bin_hash160(s))

def reverse_hash(hash, hex_format=True):
    """ hash is in hex or binary format
    """
    if not hex_format:
        hash = hexlify(hash)
    return "".join(reversed([hash[i:i+2] for i in range(0, len(hash), 2)]))

def hex_to_bin_reversed(s):
    return unhexlify(s.encode('utf8'))[::-1]

def bin_to_hex_reversed(s):
    return hexlify(s[::-1])




# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

__version__ = '0.7.7'

import services
from .services import *

import transactions
from .transactions import *

import passphrases
from .passphrases import create_passphrase
from .passphrases.legacy import random_160bit_passphrase, random_256bit_passphrase

from .b58check import b58check_encode, b58check_decode, b58check_unpack, \
    b58check_version_byte, is_b58check
from .hash import bin_hash160, hex_hash160, reverse_hash, bin_double_sha256, \
    hex_to_bin_reversed, bin_to_hex_reversed
from .formatcheck import is_secret_exponent, is_256bit_hex_string, is_wif_pk, \
    is_b58check_address, is_hex_ecdsa_pubkey, is_binary_ecdsa_pubkey
from .wallet import SDWallet
from .address import bin_hash160_to_address, script_hex_to_address, \
    hex_hash160_to_address
from .merkle import MerkleTree, calculate_merkle_pairs, calculate_merkle_root

from .keypair import *
from .privatekey import *
from .publickey import *
from .constants import *



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~
    
    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import os, json, binascii, ecdsa, hashlib

from .privatekey import random_secret_exponent
from .b58check import b58check_encode, b58check_decode, b58check_unpack, \
    b58check_version_byte
from .errors import _errors
from .hash import bin_hash160
from .formatcheck import is_int, is_256bit_hex_string, is_wif_pk, \
    is_secret_exponent
from .passphrases import create_passphrase

class BitcoinKeypair():
    """ NOTE: This object has been replaced by the BitcoinPrivateKey and 
        BitcoinPublicKey objects and is set to be deprecated at a future date.
    """

    _curve = ecdsa.curves.SECP256k1
    _hash_function = hashlib.sha256
    _pubkeyhash_version_byte = 0

    @classmethod
    def version_byte(cls, type='pubkey_hash'):
        if type == 'pubkey_hash':
            return cls._pubkeyhash_version_byte
        elif type == 'private_key':
            return (cls._pubkeyhash_version_byte + 128) % 256
        else:
            raise Exception("type must be 'pubkey_hash' or 'privatekey'")

    def __init__(self, private_key=None):
        """ Takes in a private key/secret exponent.
        """
        if not private_key:
            secret_exponent = random_secret_exponent(self._curve.order)
        elif is_int(private_key):
            secret_exponent = private_key
        elif is_256bit_hex_string(private_key):
            secret_exponent = int(private_key, 16)
        elif is_wif_pk(private_key):
            secret_exponent = int(binascii.hexlify(b58check_decode(private_key)), 16)

        # make sure that: 1 <= secret_exponent < curve_order
        if not is_secret_exponent(secret_exponent, self._curve.order):
            raise IndexError(_errors["EXPONENT_OUTSIDE_CURVE_ORDER"])
        
        self._ecdsa_private_key = ecdsa.keys.SigningKey.from_secret_exponent(
            secret_exponent, self._curve, self._hash_function
        )
        self._ecdsa_public_key = self._ecdsa_private_key.get_verifying_key()

    @classmethod
    def from_private_key(cls, private_key=None):
        return cls(private_key)

    @classmethod
    def from_passphrase(cls, passphrase=None):
        """ Create keypair from a passphrase input (a brain wallet keypair)."""
        if not passphrase:
            # run a rejection sampling algorithm to ensure the private key is
            # less than the curve order
            while True:
                passphrase = create_passphrase(bits_of_entropy=160)
                hex_private_key = hashlib.sha256(passphrase).hexdigest()
                if int(hex_private_key, 16) < cls._curve.order:
                    break
        else:
            hex_private_key = hashlib.sha256(passphrase).hexdigest()
            if not (int(hex_private_key, 16) < cls._curve.order):
                raise ValueError(_errors["PHRASE_YIELDS_INVALID_EXPONENT"])

        keypair = cls(hex_private_key)
        keypair._passphrase = passphrase
        return keypair

    def _bin_private_key(self):
        return self._ecdsa_private_key.to_string()

    def _bin_public_key(self, prefix=True):
        ecdsa_public_key = self._ecdsa_public_key.to_string()
        if prefix:
            return '\x04' + ecdsa_public_key
        return ecdsa_public_key

    def _bin_hash160(self):
        return bin_hash160(self._bin_public_key())

    def private_key(self, format='hex'):
        if format == 'bin':
            return self._bin_private_key()
        elif format == 'hex':
            return binascii.hexlify(self._bin_private_key())
        elif format == 'wif' or format == 'b58check':
            return b58check_encode(self._bin_private_key(),
                version_byte=self.version_byte('private_key'))
        else:
            raise ValueError(_errors["MUST_BE_VALID_PRIVKEY_FORMAT"])

    def public_key(self, format='hex'):
        if format == 'bin':
            return self._bin_public_key()
        elif format == 'hex':
            return binascii.hexlify(self._bin_public_key())
        else:
            raise ValueError(_errors["MUST_BE_VALID_PUBKEY_FORMAT"])

    def hash160(self, format='hex'):
        if format == 'bin':
            return self._bin_hash160()
        elif format == 'hex':
            return binascii.hexlify(self._bin_hash160())
        elif format == 'b58check':
            return b58check_encode(self._bin_hash160(),
                version_byte=self.version_byte('pubkey_hash'))
        else:
            raise ValueError(_errors["MUST_BE_VALID_HASH160_FORMAT"])

    def secret_exponent(self):
        """ The secret exponent is the private key in int or hex format. """
        return self.private_key('hex')

    def wif_pk(self):
        """ The "wif pk" is the private key in wallet import format. """
        return self.private_key('wif')

    def address(self):
        """ The address is the hash160 in b58check format. """
        return self.hash160('b58check')

    """ Brain wallet address methods """

    def passphrase(self):
        if hasattr(self, '_passphrase'):
            return self._passphrase
        else:
            raise Exception(_errors["NOT_A_BRAIN_WALLET"])

class LitecoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 48

class NamecoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 52

class PeercoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 55

class PrimecoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 23

class DogecoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 30

class WorldcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 73

class FeathercoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 14

class TerracoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 0

class NovacoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 8

class IxcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 138

class TestnetKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 111

class ProtosharesKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 56

class MemorycoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 50

class QuarkcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 58

class InfinitecoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 102

class CryptogenicbullionKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 11

class AnoncoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 23

class MegacoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 50

class EarthcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 93

class NetcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 112

class HuntercoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 40

class VertcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 71
    
class ReddcoinKeypair(BitcoinKeypair):
    _pubkeyhash_version_byte = 61

# TO DO:
# auroracoin
# counterparty
# darkcoin
# ybcoin
# maxcoin
# mintcoin
# devcoin
# tickets
# freicoin
# zetacoin
# digitalcoin
# copperlark
# applecoin
# unobtanium
# fedoracoin
# cachecoin
# mincoin
# ultracoin
# colossuscoin
# blackcoin
# securecoin
# gridcoin
# billioncoin
# kittehcoin
# karmacoin
# mooncoin
# sexcoin







# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from .opcodes import *
from .utils import count_bytes
from ..constants import MAX_BYTES_AFTER_OP_RETURN
from ..b58check import b58check_decode, b58check_encode
from binascii import hexlify, unhexlify
from utilitybelt import is_hex

def script_to_hex(script):
    """ Parse the string representation of a script and return the hex version.
        Example: "OP_DUP OP_HASH160 c629...a6db OP_EQUALVERIFY OP_CHECKSIG"
    """
    hex_script = ''
    parts = script.split(' ')
    for part in parts:
        if part[0:3] == 'OP_':
            try:
                hex_script += '%0.2x' % eval(part)
            except:
                raise Exception('Invalid opcode: %s' % part)
        elif isinstance(part, (int)):
            hex_script += '%0.2x' % part
        elif is_hex(part):
            hex_script += '%0.2x' % count_bytes(part) + part
        else:
            raise Exception('Invalid script - only opcodes and hex characters allowed.')
    return hex_script

def make_pay_to_address_script(address):
    """ Takes in an address and returns the script 
    """
    hash160 = hexlify(b58check_decode(address))
    script_string = 'OP_DUP OP_HASH160 %s OP_EQUALVERIFY OP_CHECKSIG' % hash160
    return script_to_hex(script_string)

def make_op_return_script(data, format='bin'):
    """ Takes in raw ascii data to be embedded and returns a script.
    """
    if format == 'hex':
        assert(is_hex(data))
        hex_data = data
    elif format == 'bin':
        hex_data = hexlify(data)
    else:
        raise Exception("Format must be either 'hex' or 'bin'")

    num_bytes = count_bytes(hex_data)
    if num_bytes > MAX_BYTES_AFTER_OP_RETURN:
        raise Exception('Data is %i bytes - must not exceed 40.' % num_bytes)

    script_string = 'OP_RETURN %s' % hex_data
    return script_to_hex(script_string)




from pybitcointools import mktx as make_tx
from pybitcointools import mk_multisig_script as make_multisig_script
from pybitcointools import sign as sign_tx
from pybitcointools import p2sh_scriptaddr as p2sh_script_address
from coinkit import BitcoinPrivateKey, BitcoinPublicKey

from coinkit.transactions import get_unspents

def create_unsigned_multisig_tx(hex_public_keys, inputs, send_amount, change_address,
                                tx_fee=1000, m=2, n=3):
    public_keys = [BitcoinPublicKey(k) for k in hex_public_keys]
    addresses = [public_key.address() for public_key in public_keys]

    multisig_script = make_multisig_script(hex_public_keys, m, n)
    multisig_address = p2sh_script_address(multisig_script)

    total_in = sum([input['value'] for input in inputs])
    change_amount = total_in - send_amount - tx_fee

    unsigned_tx = make_tx(
        inputs,
        [multisig_address + ':' + str(send_amount), change_address + ':' + str(change_amount)]
    )

    return unsigned_tx

def create_unsigned_address_tx(receive_address, inputs, send_amount,
                               change_address, tx_fee=1000):
    #output_script = mk_pubkey_script(receive_address)
    total_in = sum([input['value'] for input in inputs])
    change_amount = total_in - send_amount - tx_fee
    unsigned_tx = make_tx(
        inputs,
        [receive_address + ':' + str(send_amount), change_address + ':' + str(change_amount)]
    )
    return unsigned_tx

def get_inputs_formatted_for_pybitcointools(address, api='chain', auth=None):
    if api == 'chain':
        unspents = get_unspents(address, auth=auth)
    else:
        raise Exception('API not supported')

    return [{
        'output': unspent['transaction_hash'] + ':' + str(unspent['output_index']),
        'value': unspent['value']
    } for unspent in unspents]

def main():
    chain_auth = ('e21be6b69908544615b1d00f2c333c13', 'ef4c57ddb65cb5997c9f5e25b09cf520')
    
    #m = 2
    #n = 3
    send_amount = 1000

    spend_hex_private_key = '0baab1b8c6e1fef90ff9d683d2d3d4c716752966dc0ffb8886a4bff211de1adb'
    spend_address = BitcoinPrivateKey(spend_hex_private_key).public_key().address()

    inputs = get_inputs_formatted_for_pybitcointools(spend_address, auth=chain_auth)

    #multisig_private_keys = [BitcoinPrivateKey() for i in range(n)]
    #hex_multisig_private_keys = [k.to_hex() for k in multisig_private_keys]
    #print hex_multisig_private_keys
    #multisig_public_keys = [k.public_key() for k in multisig_private_keys]
    #hex_multisig_public_keys = [k.to_hex() for k in multisig_public_keys]

    unsigned_address_tx = create_unsigned_address_tx(
        '1BuD5zX3hgW7p74eouLWFWTgsaNpEo9mTG', inputs, send_amount, spend_address
    )
    print unsigned_address_tx

    #unsigned_multisig_tx = create_unsigned_multisig_tx(
    #    hex_multisig_public_keys, inputs, send_amount, spend_address, m=m, n=n)
    #print unsigned_multisig_tx
    #signed_multisig_tx = sign_tx(unsigned_multisig_tx, 0, spend_hex_private_key)
    #print signed_multisig_tx

    #print chain_com.send_transaction(signed_multisig_tx, auth=chain_auth)

main()



"""
Coinkit
==============

"""

from setuptools import setup, find_packages

setup(
    name='coinkit',
    version='0.7.7',
    url='https://github.com/onenameio/coinkit',
    license='MIT',
    author='Onename (Halfmoon Labs)',
    author_email='hello@halfmoon.io',
    description='Tools for Bitcoin & other cryptocurrencies (incl. Litecoin, Namecoin, Peercoin, Primecoin, Dogecoin, Worldcoin, Megacoin, Anoncoin, Feathercoin, Terracoin, and Novacoin).',
    keywords='bitcoin btc litecoin namecoin dogecoin cryptocurrency',
    packages=find_packages(),
    zip_safe=False,
    install_requires=[
        'ecdsa==0.11',
        'utilitybelt>=0.2.1',
        'requests>=2.4.3',
        'pybitcointools==1.1.15',
        'python-bitcoinrpc==0.1'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet',
        'Topic :: Security :: Cryptography',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# -*- coding: utf-8 -*-
"""
    Coinkit
    ~~~~~

    :copyright: (c) 2014 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

from binascii import unhexlify
from .b58check import b58check_encode

def bin_hash160_to_address(bin_hash160, version_byte=0):
    return b58check_encode(bin_hash160, version_byte=version_byte)

def hex_hash160_to_address(hash160, version_byte=0):
    return bin_hash160_to_address(unhexlify(hash160), version_byte=version_byte)

def script_hex_to_address(script, version_byte=0):
    if script[0:6] == '76a914' and script[-4:] == '88ac':
        bin_hash160 = unhexlify(script[6:-4])
        return bin_hash160_to_address(bin_hash160, version_byte=version_byte)
    return None

